window.Gleam.initSearch([{"doc":"fp","title":"fp","content":"# fp\n\n[![Package Version](https://img.shields.io/hexpm/v/fp)](https://hex.pm/packages/fp)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/fp/)\n\nHaskell-style type classes and combinators.\n\n## Example\n\nGleam provides monad-like expressions `use foo &lt;- bar(buz)` out of the box, which are very powerful. However, they can lead to scope pollution with variables. By using applicatives, we can avoid creating extra variables in many cases and adopt a more eta-reduced style:\n\n```gleam\nimport fp\nimport gleam/int\nimport gleam/base\nimport gleam/function\n\npub type Wallet {\n  Wallet(balance: Int, seed: BitString, info: String)\n}\n\npub fn applicative_web_form(\n  balance: String,\n  seed: String,\n  info: String,\n) -&gt; Result(Wallet, Nil) {\n  Wallet\n  |&gt; function.curry3\n  |&gt; fp.fmap_result(int.parse(balance))\n  |&gt; fp.ap_result(base.decode64(seed))\n  |&gt; fp.ap_result(fp.pure_result(info))\n}\n```\n\nand then:\n\n```gleam\napplicative_web_form(&quot;100&quot;, &quot;sRSOkra5qC0=&quot;, &quot;Hello!&quot;)\n// Ok(Wallet(100, &lt;&lt;177, 20, 142, 146, 182, 185, 168, 45&gt;&gt;, &quot;Hello!&quot;))\n\napplicative_web_form(&quot;WORLD!&quot;, &quot;sRSOkra5qC0=&quot;, &quot;Hello!&quot;)\n// Error(Nil)\n\napplicative_web_form(&quot;100&quot;, &quot;WORLD!&quot;, &quot;Hello!&quot;)\n// Error(Nil)\n\napplicative_web_form(&quot;WORLD!&quot;, &quot;WORLD!&quot;, &quot;Hello!&quot;)\n// Error(Nil)\n\napplicative_web_form(&quot;WORLD!&quot;, &quot;WORLD!&quot;, &quot;WORLD!&quot;)\n// Error(Nil)\n```\n\n## Expansibility\n\nCombinators like `fp.ap_result` are just shortcuts around more generic `fp.ap` with hardcoded instances:\n\n```gleam\npub fn ap_result(lhs: Result(fn(a) -&gt; b, e), rhs: Result(a, e)) -&gt; Result(b, e) {\n  ap(lhs, rhs, result_applicative_instance)\n}\n```\n\nYou can implement your own instances for other types and utilize `fp.ap`, or a simple shortcut wrapper like `fp.ap_mytype` to use applicative notation for working with your own types.\n","url":"index.html"},{"doc":"fp","title":"ApplicativeInstance","content":"pub type ApplicativeInstance(a, b, fa, fb, fab) {\n  ApplicativeInstance(\n    functor_instance: fn() -&gt; FunctorInstance(a, b, fa, fb),\n    pure: fn(a) -&gt; fa,\n    ap: fn(fab, fa) -&gt; fb,\n  )\n}\n\nApplicativeInstance(\n  functor_instance: fn() -&gt; FunctorInstance(a, b, fa, fb),\n  pure: fn(a) -&gt; fa,\n  ap: fn(fab, fa) -&gt; fb,\n)\n\n\nSynonyms:\nfp.ApplicativeInstance\nfp ApplicativeInstance","url":"fp.html#ApplicativeInstance"},{"doc":"fp","title":"FunctorInstance","content":"pub type FunctorInstance(a, b, fa, fb) {\n  FunctorInstance(fmap: fn(fn(a) -&gt; b, fa) -&gt; fb)\n}\n\nFunctorInstance(fmap: fn(fn(a) -&gt; b, fa) -&gt; fb)\n\n\nSynonyms:\nfp.FunctorInstance\nfp FunctorInstance","url":"fp.html#FunctorInstance"},{"doc":"fp","title":"ap","content":"pub fn ap(lhs: a, rhs: b, instance: fn() -&gt;\n    ApplicativeInstance(c, d, b, e, a)) -&gt; e\n\nSynonyms:\nfp.ap\nfp ap","url":"fp.html#ap"},{"doc":"fp","title":"ap_list","content":"pub fn ap_list(lhs: List(fn(a) -&gt; b), rhs: List(a)) -&gt; List(b)\n\nSynonyms:\nfp.ap_list\nfp ap_list","url":"fp.html#ap_list"},{"doc":"fp","title":"ap_option","content":"pub fn ap_option(lhs: Option(fn(a) -&gt; b), rhs: Option(a)) -&gt; Option(\n  b,\n)\n\nSynonyms:\nfp.ap_option\nfp ap_option","url":"fp.html#ap_option"},{"doc":"fp","title":"ap_result","content":"pub fn ap_result(lhs: Result(fn(a) -&gt; b, c), rhs: Result(a, c)) -&gt; Result(\n  b,\n  c,\n)\n\nSynonyms:\nfp.ap_result\nfp ap_result","url":"fp.html#ap_result"},{"doc":"fp","title":"fmap","content":"pub fn fmap(lhs: fn(a) -&gt; b, rhs: c, instance: fn() -&gt;\n    FunctorInstance(a, b, c, d)) -&gt; d\n\nSynonyms:\nfp.fmap\nfp fmap","url":"fp.html#fmap"},{"doc":"fp","title":"fmap_function","content":"pub fn fmap_function(lhs: fn(a) -&gt; b, rhs: fn(c) -&gt; a) -&gt; fn(c) -&gt;\n  b\n\nSynonyms:\nfp.fmap_function\nfp fmap_function","url":"fp.html#fmap_function"},{"doc":"fp","title":"fmap_list","content":"pub fn fmap_list(lhs: fn(a) -&gt; b, rhs: List(a)) -&gt; List(b)\n\nSynonyms:\nfp.fmap_list\nfp fmap_list","url":"fp.html#fmap_list"},{"doc":"fp","title":"fmap_option","content":"pub fn fmap_option(lhs: fn(a) -&gt; b, rhs: Option(a)) -&gt; Option(b)\n\nSynonyms:\nfp.fmap_option\nfp fmap_option","url":"fp.html#fmap_option"},{"doc":"fp","title":"fmap_result","content":"pub fn fmap_result(lhs: fn(a) -&gt; b, rhs: Result(a, c)) -&gt; Result(\n  b,\n  c,\n)\n\nSynonyms:\nfp.fmap_result\nfp fmap_result","url":"fp.html#fmap_result"},{"doc":"fp","title":"function_functor_instance","content":"pub fn function_functor_instance() -&gt; FunctorInstance(\n  a,\n  b,\n  fn(c) -&gt; a,\n  fn(c) -&gt; b,\n)\n\nSynonyms:\nfp.function_functor_instance\nfp function_functor_instance","url":"fp.html#function_functor_instance"},{"doc":"fp","title":"list_applicative_instance","content":"pub fn list_applicative_instance() -&gt; ApplicativeInstance(\n  a,\n  b,\n  List(a),\n  List(b),\n  List(fn(a) -&gt; b),\n)\n\nSynonyms:\nfp.list_applicative_instance\nfp list_applicative_instance","url":"fp.html#list_applicative_instance"},{"doc":"fp","title":"list_functor_instance","content":"pub fn list_functor_instance() -&gt; FunctorInstance(\n  a,\n  b,\n  List(a),\n  List(b),\n)\n\nSynonyms:\nfp.list_functor_instance\nfp list_functor_instance","url":"fp.html#list_functor_instance"},{"doc":"fp","title":"new_functor_instance","content":"pub fn new_functor_instance(fmapper: fn(a, fn(b) -&gt; c) -&gt; d) -&gt; FunctorInstance(\n  b,\n  c,\n  a,\n  d,\n)\n\nSynonyms:\nfp.new_functor_instance\nfp new_functor_instance","url":"fp.html#new_functor_instance"},{"doc":"fp","title":"option_applicative_instance","content":"pub fn option_applicative_instance() -&gt; ApplicativeInstance(\n  a,\n  b,\n  Option(a),\n  Option(b),\n  Option(fn(a) -&gt; b),\n)\n\nSynonyms:\nfp.option_applicative_instance\nfp option_applicative_instance","url":"fp.html#option_applicative_instance"},{"doc":"fp","title":"option_functor_instance","content":"pub fn option_functor_instance() -&gt; FunctorInstance(\n  a,\n  b,\n  Option(a),\n  Option(b),\n)\n\nSynonyms:\nfp.option_functor_instance\nfp option_functor_instance","url":"fp.html#option_functor_instance"},{"doc":"fp","title":"pure","content":"pub fn pure(x: a, instance: fn() -&gt;\n    ApplicativeInstance(a, b, c, d, e)) -&gt; c\n\nSynonyms:\nfp.pure\nfp pure","url":"fp.html#pure"},{"doc":"fp","title":"pure_list","content":"pub fn pure_list(x: a) -&gt; List(a)\n\nSynonyms:\nfp.pure_list\nfp pure_list","url":"fp.html#pure_list"},{"doc":"fp","title":"pure_option","content":"pub fn pure_option(x: a) -&gt; Option(a)\n\nSynonyms:\nfp.pure_option\nfp pure_option","url":"fp.html#pure_option"},{"doc":"fp","title":"pure_result","content":"pub fn pure_result(x: a) -&gt; Result(a, b)\n\nSynonyms:\nfp.pure_result\nfp pure_result","url":"fp.html#pure_result"},{"doc":"fp","title":"result_applicative_instance","content":"pub fn result_applicative_instance() -&gt; ApplicativeInstance(\n  a,\n  b,\n  Result(a, c),\n  Result(b, c),\n  Result(fn(a) -&gt; b, c),\n)\n\nSynonyms:\nfp.result_applicative_instance\nfp result_applicative_instance","url":"fp.html#result_applicative_instance"},{"doc":"fp","title":"result_functor_instance","content":"pub fn result_functor_instance() -&gt; FunctorInstance(\n  a,\n  b,\n  Result(a, c),\n  Result(b, c),\n)\n\nSynonyms:\nfp.result_functor_instance\nfp result_functor_instance","url":"fp.html#result_functor_instance"},{"doc":"fp","title":"fp","content":"","url":"fp.html"}]);